// CSCE 4620 - Assignment 1 (Timers)
// Jeff Morrison
//
// NOTE: Overall OS code-structure based on Example1 from 
//         the uC/OS-II CD.

#include "includes.h"

#define  TASK_STK_SIZE                 512            /* Size of each task's stacks (# of WORDs)       */
#define  N_TASKS                        2             /* Number of tasks                               */

OS_STK        TaskStk[N_TASKS][TASK_STK_SIZE];        /* Tasks stacks                                  */
OS_STK        TaskStartStk[TASK_STK_SIZE];
char          TaskData[N_TASKS];                      /* Parameters to pass to each task               */
INT8U         StatTickTime;

        void  TimerTask(void *pdata);                  /* Function prototypes of timer-tasks            */
        void  TaskStart(void *data);                  /* Function prototypes of Startup task           */
static  void  TaskStartDispInit(void);

void main(void) {
	
    PC_DispClrScr(DISP_FGND_WHITE + DISP_BGND_BLACK);      /* Clear the screen                         */

    OSInit();                                              /* Initialize uC/OS-II                      */

    PC_DOSSaveReturn();                                    /* Save environment to return to DOS        */
    PC_VectSet(uCOS, OSCtxSw);                             /* Install uC/OS-II's context switch vector */

    OSTaskCreate(TaskStart, (void *)0, &TaskStartStk[TASK_STK_SIZE - 1], 0);
    OSStart();                                             /* Start multitasking                       */
}

void TaskStart(void *pdata) {
	
#if OS_CRITICAL_METHOD == 3                                /* Allocate storage for CPU status register */
    OS_CPU_SR  cpu_sr;
#endif
    char       s[100];
    INT16S     key;

    pdata = pdata;                                         /* Prevent compiler warning                 */

    TaskStartDispInit();                                   /* Initialize the display                   */

    OS_ENTER_CRITICAL();
    PC_VectSet(0x08, OSTickISR);                           /* Install uC/OS-II's clock tick ISR        */
    PC_SetTickRate(OS_TICKS_PER_SEC);                      /* Reprogram tick rate                      */
    OS_EXIT_CRITICAL();

    OSStatInit();                                          /* Initialize uC/OS-II's statistics         */
	
	//INIT TICK-STAT VALUE TO 0
	StatTickTime = 0;

	//CREATE TASK TO RECORD STAT TIMER
	OSTaskCreate(TimerTask, (void *)&TaskData[1], &TaskStk[1][TASK_STK_SIZE - 1], 1);

    for (;;) {
	
	TaskStartDispInit();
	
        if (PC_GetKey(&key) == TRUE) {                     /* See if key has been pressed              */
            if (key == 0x1B) {                             /* Yes, see if it's the ESCAPE key          */
                PC_DOSReturn();                            /* Return to DOS                            */
            }
        }

        OSCtxSwCtr = 0;                                    /* Clear context switch counter             */
        OSTimeDlyHMSM(0, 0, 0, 500);                         /* Wait 500 ms                          */
    }
}

static  void  TaskStartDispInit(void) {

	PC_DispStr( 0,  0, "                                    timer.c                                     ", DISP_FGND_WHITE + DISP_BGND_RED);
    PC_DispStr( 0,  1, "                                 Jeff Morrison                                  ", DISP_FGND_BLACK + DISP_BGND_LIGHT_GRAY);
    PC_DispStr( 0,  2, "                                                                                ", DISP_FGND_BLACK + DISP_BGND_LIGHT_GRAY);
    PC_DispStr( 0,  3, "                                                                                ", DISP_FGND_BLACK + DISP_BGND_LIGHT_GRAY);
    PC_DispStr( 0,  4, "                                                                                ", DISP_FGND_BLACK + DISP_BGND_LIGHT_GRAY);
    PC_DispStr( 0,  5, "                                                                                ", DISP_FGND_BLACK + DISP_BGND_LIGHT_GRAY);
    PC_DispStr( 0,  6, "                                                                                ", DISP_FGND_BLACK + DISP_BGND_LIGHT_GRAY);
    PC_DispStr( 0,  7, "                                                                                ", DISP_FGND_BLACK + DISP_BGND_LIGHT_GRAY);
    PC_DispStr( 0,  8, "                                                                                ", DISP_FGND_BLACK + DISP_BGND_LIGHT_GRAY);
	PC_DispStr( 0,  9, "                    Actual Clock Rate:                                          ", DISP_FGND_BLACK + DISP_BGND_LIGHT_GRAY);
	PC_DispStr( 0, 10, "          Statistical Tick Delay Time:                                          ", DISP_FGND_BLACK + DISP_BGND_LIGHT_GRAY);
    PC_DispStr( 0, 11, "                                                                                ", DISP_FGND_BLACK + DISP_BGND_LIGHT_GRAY);
    PC_DispStr( 0, 12, "                                                                                ", DISP_FGND_BLACK + DISP_BGND_LIGHT_GRAY);
    PC_DispStr( 0, 13, "                                                                                ", DISP_FGND_BLACK + DISP_BGND_LIGHT_GRAY);
    PC_DispStr( 0, 14, "                                                                                ", DISP_FGND_BLACK + DISP_BGND_LIGHT_GRAY);
    PC_DispStr( 0, 15, "                                                                                ", DISP_FGND_BLACK + DISP_BGND_LIGHT_GRAY);
    PC_DispStr( 0, 16, "                                                                                ", DISP_FGND_BLACK + DISP_BGND_LIGHT_GRAY);
    PC_DispStr( 0, 17, "                                                                                ", DISP_FGND_BLACK + DISP_BGND_LIGHT_GRAY);
    PC_DispStr( 0, 18, "                                                                                ", DISP_FGND_BLACK + DISP_BGND_LIGHT_GRAY);
    PC_DispStr( 0, 19, "                                                                                ", DISP_FGND_BLACK + DISP_BGND_LIGHT_GRAY);
    PC_DispStr( 0, 20, "                                                                                ", DISP_FGND_BLACK + DISP_BGND_LIGHT_GRAY);
    PC_DispStr( 0, 21, "                                                                                ", DISP_FGND_BLACK + DISP_BGND_LIGHT_GRAY);
    PC_DispStr( 0, 21, "                                                                                ", DISP_FGND_BLACK + DISP_BGND_LIGHT_GRAY);
    PC_DispStr( 0, 21, "                                                                                ", DISP_FGND_BLACK + DISP_BGND_LIGHT_GRAY);
    PC_DispStr( 0, 24, "                            <-PRESS 'ESC' TO QUIT->                             ", DISP_FGND_BLACK + DISP_BGND_LIGHT_GRAY + DISP_BLINK);
}

static void TimerTask(void *pdata) {
	
	INT16U elapsedTime;
	INT8U tps[80];
	INT8U stt[70];
	//INT8U endTime;
	
	for (;;) {
		
		//INIT TIMER
		PC_ElapsedInit();
		
		//STORE START TIME
		PC_ElapsedStart();

		//DELAY FOR 1 TICK
		OSTimeDly(1);

		//STORE ELAPSED TIME
		elapsedTime = PC_ElapsedStop();
		
		//AVG TICK TIME
		if (StatTickTime == 0)
			StatTickTime = elapsedTime;
		else
			StatTickTime = (StatTickTime + elapsedTime)/2;
		
		sprintf(tps, "%08x", OS_TICKS_PER_SEC);
		sprintf(stt, "%08x", elapsedTime);
		
		//DISPLAY ACTUAL TICK TIME
		PC_DispChar(40, 9, (char) tps, DISP_FGND_BLACK + DISP_BGND_LIGHT_GRAY);
		//PC_DispChar(40, 9, 'a', DISP_FGND_BLACK + DISP_BGND_LIGHT_GRAY);

		//DISPLAY STAT TICK TIME
		PC_DispChar(40, 10, (char) stt, DISP_FGND_BLACK + DISP_BGND_LIGHT_GRAY);
		//PC_DispChar(40, 10, 'b', DISP_FGND_BLACK + DISP_BGND_LIGHT_GRAY);
	}
}
